@@ -5,6 +5,7 @@ from typing import TYPE_CHECKING, Any, Callable, Coroutine, Tuple, Union
 
 from rap.client.model import Request, Response
 from rap.common.channel import BaseChannel
+from rap.common.conn import Connection
 from rap.common.exceptions import ChannelError
 from rap.common.utlis import Constant
 
@@ -12,6 +13,9 @@ if TYPE_CHECKING:
     from rap.client.transoprt.transport import Session
 
 
+__all__ = ["Channel"]
+
+
 class Channel(BaseChannel):
     """support channel use"""
 
@@ -21,33 +25,37 @@ class Channel(BaseChannel):
         session: "Session",
         create: Callable[[str], Coroutine[Any, Any, Any]],
         read: Callable[[str], Coroutine[Any, Any, Response]],
-        write: Callable[[Request], Coroutine[Any, Any, str]],
+        write: Callable[[Request, "Session"], Coroutine[Any, Any, None]],
         close: Callable[[str], Coroutine[Any, Any, Any]],
+        add_listen_conn_exc: Callable[[str, Connection], None],
     ):
         self.channel_id: str = str(uuid.uuid4())
         self._func_name: str = fun_name
         self._session: "Session" = session
         self._create: Callable[[str], Coroutine[Any, Any, Any]] = create
         self._read: Callable[[str], Coroutine[Any, Any, Response]] = read
-        self._write: Callable[[Request], Coroutine[Any, Any, str]] = write
+        self._write: Callable[[Request, "Session"], Coroutine[Any, Any, None]] = write
         self._close: Callable[[str], Coroutine[Any, Any, Any]] = close
+        self._add_listen_conn_exc: Callable[[str, Connection], None] = add_listen_conn_exc
         self._is_close: bool = True
 
     async def create(self):
-        """create and init channel, create session"""
+        """create and init channel, create session and listen conn exc"""
         if not self._is_close:
             raise ChannelError("channel already create")
+
+        # init channel data structure
         self._session.create()
         await self._create(self.channel_id)
+        self._add_listen_conn_exc(self.channel_id, self._session.conn)
         self._is_close = False
 
-        life_cycle: str = "declare"
+        # init with server
+        life_cycle: str = Constant.DECLARE
         await self._base_write(None, life_cycle)
         response: Response = await self._base_read()
         if response.header.get("channel_life_cycle") != life_cycle:
             raise ChannelError("channel life cycle error")
-        channel_id: str = response.header.get("channel_id")
-        self.channel_id = channel_id
 
     async def _base_read(self) -> Response:
         """base read response msg from channel conn
@@ -56,9 +64,10 @@ class Channel(BaseChannel):
         if self._is_close:
             raise ChannelError(f"channel is closed")
         response: Union[Response, Exception] = await self._read(self.channel_id)
+
         if isinstance(response, Exception):
             raise response
-        if response.header.get("channel_life_cycle") == "drop":
+        if response.header.get("channel_life_cycle") == Constant.DROP:
             self._is_close = True
             await self._close(self.channel_id)
             raise ChannelError("recv drop event, close channel")
@@ -88,28 +97,27 @@ class Channel(BaseChannel):
         ...     while await channel.loop(cnt < 3):
         ...         pass
         """
-        await asyncio.sleep(0.01)
+        await asyncio.sleep(0)
         if self._is_close:
             return not self._is_close
         else:
             return flag
 
-    async def _base_write(self, body: Any, life_cycle: str) -> str:
+    async def _base_write(self, body: Any, life_cycle: str):
         """base send body to channel"""
         if self._is_close:
             raise ChannelError(f"channel is closed")
         request: Request = Request(
-            Constant.MSG_REQUEST,
+            Constant.CHANNEL_REQUEST,
             self._func_name,
-            Constant.CHANNEL,
             body,
             {"channel_life_cycle": life_cycle, "channel_id": self.channel_id},
         )
-        return await self._write(request)
+        await self._write(request, self._session)
 
     async def read(self) -> Response:
         response: Response = await self._base_read()
-        if response.header.get("channel_life_cycle") != "msg":
+        if response.header.get("channel_life_cycle") != Constant.MSG:
             raise ChannelError("channel life cycle error")
         return response
 
@@ -118,14 +126,13 @@ class Channel(BaseChannel):
         return response.body
 
     async def write(self, body: Any):
-        await self._base_write(body, "msg")
+        await self._base_write(body, Constant.MSG)
 
     async def close(self):
         """Actively send a close message and close the channel"""
         if self._is_close:
             return
-        self._session.close()
-        life_cycle: str = "drop"
+        life_cycle: str = Constant.DROP
         await self._base_write(None, life_cycle)
 
         async def wait_drop_response():
@@ -140,6 +147,7 @@ class Channel(BaseChannel):
             await asyncio.wait_for(wait_drop_response(), 3)
         except asyncio.TimeoutError:
             logging.warning("wait drop response timeout")
+        self._session.close()
 
     ######################
     # async with support #
@@ -151,18 +159,6 @@ class Channel(BaseChannel):
     async def __aexit__(self, *args: Tuple):
         await self.close()
 
-    #####################
-    # async for support #
-    #####################
-    def __aiter__(self) -> "Channel":
-        return self
-
-    async def __anext__(self):
-        try:
-            return await self.read_body()
-        except ChannelError:
-            raise StopAsyncIteration()
-
     @property
     def is_close(self) -> bool:
         return self._is_close
