@@ -4,7 +4,8 @@ import random
 import uuid
 from contextvars import ContextVar, Token
 from dataclasses import dataclass
-from typing import Any, Dict, List, Optional, Tuple, Type, Union
+from inspect import isfunction
+from typing import Any, Callable, Coroutine, Dict, List, Optional, Tuple, Type, Union
 
 from rap.client.model import Request, Response
 from rap.client.processor.base import BaseProcessor
@@ -12,11 +13,12 @@ from rap.client.transoprt.channel import Channel
 from rap.client.utils import get_rap_exc_dict, raise_rap_error
 from rap.common import exceptions as rap_exc
 from rap.common.conn import Connection
-from rap.common.exceptions import ProtocolError, RPCError
+from rap.common.exceptions import ChannelError, RPCError
 from rap.common.types import BASE_REQUEST_TYPE, BASE_RESPONSE_TYPE
-from rap.common.utlis import MISS_OBJECT, Constant, Event, gen_random_str_id
+from rap.common.utlis import MISS_OBJECT, Constant, Event, as_first_completed
 
-_conn_context: ContextVar[Connection] = ContextVar("conn_context", default=MISS_OBJECT)
+_session_context: ContextVar["Optional[Session]"] = ContextVar("session_context", default=MISS_OBJECT)
+__all__ = ["Session", "Transport"]
 
 
 @dataclass()
@@ -65,14 +67,10 @@ class Transport(object):
         if host in self._conn_dict:
             await self._conn_dict[host].await_close()
 
-        conn: Connection = Connection(
-            self._timeout,
-            ssl_crt_path=self._ssl_crt_path,
-        )
+        conn: Connection = Connection(self._timeout, ssl_crt_path=self._ssl_crt_path)
         ip, port = host.split(":")
         await conn.connect(ip, int(port))
         future: asyncio.Future = asyncio.ensure_future(self._listen(conn))
-        await self._declare_life_cycle(conn)
         self._conn_dict[host] = ConnModel(conn, future)
         logging.debug(f"Connection to %s...", conn.connection_info)
 
@@ -83,10 +81,7 @@ class Transport(object):
         if not self._is_close:
             raise ConnectionError(f"{self.__class__.__name__} already connect")
         for host in self._host_list:
-            if host in self._conn_dict and not self._conn_dict[host].is_closed():
-                logging.warning(f"{host} already connected")
-            else:
-                await self._connect(host)
+            await self._connect(host)
         self._is_close = False
 
     async def await_close(self):
@@ -101,7 +96,6 @@ class Transport(object):
             else:
                 conn: Connection = self._conn_dict[host].conn
                 future: asyncio.Future = self._conn_dict[host].future
-                await self._drop_life_cycle(conn)
                 if not future.cancelled():
                     future.cancel()
                 await conn.await_close()
@@ -117,6 +111,7 @@ class Transport(object):
         except asyncio.CancelledError:
             pass
         except Exception as e:
+            conn.set_reader_exc(e)
             logging.exception(f"listen status:{self._is_close} error: {e}, close conn:{conn}")
             if not conn.is_closed():
                 conn.close()
@@ -128,16 +123,13 @@ class Transport(object):
             logging.debug(f"recv raw data: %s", response_msg)
         except asyncio.TimeoutError as e:
             logging.error(f"recv response from {conn.connection_info} timeout")
-            conn.set_reader_exc(e)
             raise e
         except asyncio.CancelledError:
             return
-        except Exception as e:
-            conn.set_reader_exc(e)
-            raise e
 
         if response_msg is None:
             raise ConnectionError("Connection has been closed")
+
         # parse response
         try:
             response: Response = Response(*response_msg)
@@ -154,26 +146,24 @@ class Transport(object):
 
         resp_future_id: str = f"{conn.sock_tuple}:{response.msg_id}"
         channel_id: Optional[str] = response.header.get("channel_id")
-        if channel_id and response.method != Constant.CHANNEL and not exc:
-            exc: Exception = ProtocolError(f"recv error method:{response.method}")
+
+        def put_exc_to_receiver(put_exc: Exception):
+            if channel_id and channel_id in self._channel_queue_dict:
+                self._channel_queue_dict[channel_id].put_nowait(put_exc)
+            elif response.msg_id != -1 and resp_future_id in self._resp_future_dict:
+                self._resp_future_dict[resp_future_id].set_exception(put_exc)
+            else:
+                logging.error(f"recv error msg:{response}, ignore")
 
         if exc:
-            if channel_id in self._channel_queue_dict:
-                self._channel_queue_dict[channel_id].put_nowait(exc)
-            elif response.msg_id != -1 and resp_future_id in self._resp_future_dict:
-                self._resp_future_dict[resp_future_id].set_exception(exc)
-            else:
-                logging.error(f"recv error msg:{response}")
+            put_exc_to_receiver(exc)
             return
 
         # server error response handle
         if response.num == Constant.SERVER_ERROR_RESPONSE:
             status_code: int = response.header.get("status_code", 500)
             exc: Type["rap_exc.BaseRapError"] = self._rap_exc_dict.get(status_code, rap_exc.BaseRapError)
-            if channel_id:
-                self._channel_queue_dict[channel_id].put_nowait(exc(response.body))
-            else:
-                self._resp_future_dict[resp_future_id].set_exception(exc(response.body))
+            put_exc_to_receiver(exc(response.body))
             return
 
         # server event msg handle
@@ -183,9 +173,8 @@ class Transport(object):
                 raise RuntimeError(f"recv close conn event, event info:{event_info}")
             elif event == Constant.PING_EVENT:
                 request: Request = Request(
-                    Constant.CLIENT_EVENT_RESPONSE, "", "", Event(Constant.PONG_EVENT, "").to_tuple()
+                    Constant.CLIENT_EVENT_RESPONSE, "", Event(Constant.PONG_EVENT, "").to_tuple()
                 )
-                self.before_request_handle(request, conn)
                 await self.write(request, -1, conn)
                 return
 
@@ -203,95 +192,84 @@ class Transport(object):
             return
         self._resp_future_dict[resp_future_id].set_result(response)
 
-    ##############
-    # life cycle #
-    ##############
-    async def _declare_life_cycle(self, conn: Connection):
-        """send declare msg and init client id"""
-        random_id: str = gen_random_str_id()
-        request: Request = Request(Constant.DECLARE_REQUEST, "", Constant.NORMAL, random_id)
-        response = await self._base_request(request, conn)
-        if response.num != Constant.DECLARE_RESPONSE and response.body != random_id[::-1]:
-            raise RPCError("declare response error")
-        logging.info("declare success")
-
-    async def _drop_life_cycle(self, conn: Connection):
-        """send drop msg"""
-        random_id: str = gen_random_str_id(8)
-        request: Request = Request(Constant.DROP_REQUEST, "", Constant.NORMAL, random_id)
-        response = await self._base_request(request, conn)
-        if response.num != Constant.DROP_RESPONSE and response.body != random_id[::-1]:
-            logging.warning("drop response error")
-        else:
-            logging.info("drop response success")
-
     ####################################
     # base one by one request response #
     ####################################
-    async def _base_request(self, request: Request, conn: Connection) -> Response:
+    async def _base_request(
+        self, request: Request, conn: Optional[Connection] = None, session: Optional["Session"] = None
+    ) -> Response:
         """gen msg id, send and recv response"""
         msg_id: int = self._msg_id + 1
         # Avoid too big numbers
         self._msg_id = msg_id & 65535
 
-        resp_future_id: str = await self.write(request, msg_id, conn)
+        conn, resp_future_id = await self.write(request, msg_id, conn, session)
         try:
-            return await self.read(resp_future_id)
+            return await self.read(resp_future_id, conn)
         finally:
             if resp_future_id in self._resp_future_dict:
                 del self._resp_future_dict[resp_future_id]
 
-    def before_request_handle(self, request: Request, conn: Optional[Connection] = None) -> Connection:
-        """check conn and header"""
-        if not conn:
-            conn = self.now_conn
-        if conn.is_closed():
-            raise ConnectionError("The connection has been closed, please call connect to create connection")
+    @staticmethod
+    def before_write_handle(request: Request):
+        """check and header"""
 
         def set_header_value(header_key: str, header_Value: Any):
-            """set header value"""
+            """if key not in header, set header value"""
             if header_key not in request.header:
                 request.header[header_key] = header_Value
 
-        use_session: bool = False
-        if _conn_context.get(MISS_OBJECT) is not MISS_OBJECT:
-            use_session = True
-        request.header["use_session"] = use_session
-
         set_header_value("version", Constant.VERSION)
         set_header_value("user_agent", Constant.USER_AGENT)
         set_header_value("request_id", str(uuid.uuid4()))
-        return conn
 
     #######################
     # base write&read api #
     #######################
-    async def write(self, request: Request, msg_id: int, conn: Optional[Connection] = None) -> str:
+    async def write(
+        self, request: Request, msg_id: int, conn: Optional[Connection] = None, session: Optional["Session"] = None
+    ) -> Tuple[Connection, str]:
         """write msg to conn, If it is not a normal request, you need to set msg_id: -1"""
-        conn = self.before_request_handle(request, conn)
-        for process_request in self._process_request_list:
-            await process_request(request)
-        request_msg: BASE_REQUEST_TYPE = request.gen_request_msg(msg_id)
-        try:
-            await conn.write(request_msg)
-            logging.debug(f"send:%s to %s", request_msg, conn.connection_info)
-        except asyncio.TimeoutError as e:
-            logging.error(f"send to %s timeout, drop data:%s", conn.connection_info, request_msg)
-            raise e
-        except Exception as e:
-            raise e
+        if not session or not session.id:
+            session = _session_context.get(None)
+        if session:
+            conn = session.conn
+            request.header["session_id"] = session.id
+        elif not conn:
+            conn = self.get_random_conn()
 
-        if "channel_id" in request.header:
-            return request.header["channel_id"]
+        async def _write():
+            self.before_write_handle(request)
+
+            for process_request in self._process_request_list:
+                await process_request(request)
+
+            request_msg: BASE_REQUEST_TYPE = request.gen_request_msg(msg_id)
+            try:
+                await conn.write(request_msg)
+            except asyncio.TimeoutError as e:
+                logging.error(f"send to %s timeout, drop data:%s", conn.connection_info, request_msg)
+                raise e
+            except Exception as e:
+                raise e
+
+        if request.num == Constant.CHANNEL_REQUEST:
+            if "channel_id" not in request.header:
+                raise ChannelError("not found channel id in header")
+            await _write()
+            return conn, request.header["channel_id"]
         else:
+            await _write()
             resp_future_id: str = f"{conn.sock_tuple}:{msg_id}"
             self._resp_future_dict[resp_future_id] = asyncio.Future()
-            return resp_future_id
+            return conn, resp_future_id
 
-    async def read(self, resp_future_id: str) -> Response:
+    async def read(self, resp_future_id: str, conn: Connection) -> Response:
         """write response msg(except channel response)"""
         try:
-            return await asyncio.wait_for(self._resp_future_dict[resp_future_id], self._timeout)
+            return await as_first_completed(
+                [asyncio.wait_for(self._resp_future_dict[resp_future_id], self._timeout), conn.result_future]
+            )
         except asyncio.TimeoutError:
             raise asyncio.TimeoutError(f"msg_id:{resp_future_id} request timeout")
 
@@ -299,15 +277,21 @@ class Transport(object):
     # one by one request #
     ######################
     async def request(
-        self, func_name: str, *args, call_id=-1, conn: Optional[Connection] = None, header: Optional[dict] = None
+        self,
+        func_name: str,
+        *args,
+        call_id=-1,
+        conn: Optional[Connection] = None,
+        group: str = "normal",
+        header: Optional[dict] = None,
+        session: Optional["Session"] = None,
     ) -> Response:
         """msg request handle"""
-        request: Request = Request(
-            Constant.MSG_REQUEST, func_name, Constant.NORMAL, {"call_id": call_id, "param": args}
-        )
-        if header is not None:
+        # if len([i for i in args if i is not MISS_OBJECT]) > 0
+        request: Request = Request(Constant.MSG_REQUEST, func_name, {"call_id": call_id, "param": args, "group": group})
+        if header:
             request.header.update(header)
-        response: Response = await self._base_request(request, conn)
+        response: Response = await self._base_request(request, conn=conn, session=session)
         if response.num != Constant.MSG_RESPONSE:
             raise RPCError("request num error")
         if "exc" in response.body:
@@ -324,13 +308,6 @@ class Transport(object):
         key: str = random.choice(self._host_list)
         return self._conn_dict[key].conn
 
-    @property
-    def now_conn(self) -> Connection:
-        conn: Optional[Connection] = _conn_context.get()
-        if conn is MISS_OBJECT:
-            conn = self.get_random_conn()
-        return conn
-
     @property
     def session(self) -> "Session":
         return Session(self)
@@ -342,18 +319,23 @@ class Transport(object):
         async def read(_channel_id: str) -> Response:
             return await self._channel_queue_dict[_channel_id].get()
 
-        async def write(request: Request) -> str:
-            conn = self.before_request_handle(request)
-            return await self.write(request, -1, conn)
+        async def write(request: Request, session: "Session"):
+            await self.write(request, -1, session=session)
 
         async def close(_call_id: str):
             del self._channel_queue_dict[_call_id]
 
-        return Channel(func_name, self.session, create, read, write, close)
+        def listen_conn_exc(_channel_id: str, conn: Connection):
+            async def _add_exc_queue(exc: Exception):
+                await self._channel_queue_dict[_channel_id].put(exc)
 
-    ##############
+            conn.add_listen_func(_add_exc_queue)
+
+        return Channel(func_name, self.session, create, read, write, close, listen_conn_exc)
+
+    #############
     # processor #
-    ##############
+    #############
     def load_processor(self, processor_list: List[BaseProcessor]):
         for middleware in processor_list:
             self._process_request_list.append(middleware.process_request)
@@ -368,6 +350,8 @@ class Session(object):
     def __init__(self, transport: "Transport"):
         self._transport: "Transport" = transport
         self._token: Optional[Token] = None
+        self.id: Optional[str] = None
+        self._conn: Optional[Connection] = None
 
     async def __aenter__(self) -> "Session":
         self.create()
@@ -377,27 +361,50 @@ class Session(object):
         self.close()
 
     def create(self):
-        self._token: Token = _conn_context.set(self._transport.get_random_conn())
+        self.id = str(uuid.uuid4())
+        self._conn = self._transport.get_random_conn()
+        self._token: Token = _session_context.set(self)
 
     def close(self):
-        _conn_context.reset(self._token)
+        self.id = None
+        self._conn = None
+        _session_context.reset(self._token)
 
     @property
     def in_session(self) -> bool:
-        return _conn_context.get(MISS_OBJECT) is not MISS_OBJECT
+        return _session_context.get(MISS_OBJECT) is not MISS_OBJECT
 
     @property
     def conn(self) -> Connection:
-        conn: Optional[Connection] = _conn_context.get()
-        if conn is MISS_OBJECT:
-            raise RuntimeError("Session has not been created")
-        return conn
+        return self._conn
 
-    async def request(self, method: str, *args, call_id=-1) -> Response:
-        return await self._transport.request(method, *args, call_id, self.conn)
+    async def request(self, method: str, *args, call_id=-1, header: Optional[dict] = None) -> Any:
+        return await self._transport.request(
+            method, *args, call_id=call_id, conn=self.conn, header=header, session=self
+        )
 
-    async def write(self, request: Request, msg_id: int) -> str:
-        return await self._transport.write(request, msg_id, self.conn)
+    async def write(self, request: Request, msg_id: int):
+        await self._transport.write(request, msg_id, session=self)
 
     async def read(self, resp_future_id: str) -> Response:
-        return await self._transport.read(resp_future_id)
+        return await self._transport.read(resp_future_id, self.conn)
+
+    async def execute(
+        self,
+        obj: Union[Callable, Coroutine, str],
+        arg_list: Optional[List] = None,
+        kwarg_dict: Optional[Dict[str, Any]] = None,
+    ):
+        if asyncio.iscoroutine(obj):
+            assert obj.cr_frame.f_locals["self"].transport is self._transport
+            obj.cr_frame.f_locals["kwargs"]["session"] = self
+        elif isfunction(obj) and arg_list:
+            kwarg_dict = kwarg_dict if kwarg_dict else {}
+            response: Response = await self.request(obj.__name__, *arg_list, **kwarg_dict)
+            return response.body["result"]
+        elif type(obj) is str and arg_list:
+            kwarg_dict = kwarg_dict if kwarg_dict else {}
+            response: Response = await self.request(obj, *arg_list, **kwarg_dict)
+            return response.body["result"]
+        else:
+            raise TypeError(f"Not support {type(obj)}, obj type must: {Callable}, {Coroutine}, {str}")
