@@ -179,7 +179,6 @@ class Server(object):
                 return
             try:
                 request: RequestModel = RequestModel(*_request_msg)
-                request.header["_host"] = conn.peer_tuple
             except Exception as closer_e:
                 logging.error(f"{conn.peer_tuple} send bad msg:{_request_msg}, error:{closer_e}")
                 await response_handle(ResponseModel(body=Event(Constant.EVENT_CLOSE_CONN, "protocol error")))
@@ -202,8 +201,7 @@ class Server(object):
                 logging.error(f"recv data from {conn.peer_tuple} timeout. close conn")
                 await response_handle(ResponseModel(body=Event(Constant.EVENT_CLOSE_CONN, "keep alive timeout")))
                 break
-            except IOError as e:
-                logging.debug(f"close conn:%s info:%s", conn.peer_tuple, e)
+            except IOError:
                 break
             except Exception as e:
                 logging.error(f"recv data from {conn.peer_tuple} error:{e}, conn has been closed")
